﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Net;
using System.Net.Http;
using System.Security.Policy;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using Newtonsoft.Json;
using Newtonsoft.Json.Linq;

namespace 幻能边境anima下载工具
{
    internal class Program
    {
        const string BASE64_KEYS = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/"; // 请替换为实际值
        const string HEX_CHARS = "0123456789abcdef"; // 十六进制字符
        const string LogName = "@错误日志.txt";
        private static readonly HttpClient _httpClient = new HttpClient();
        private static readonly object _fileLock = new object();
        private static readonly SemaphoreSlim _downloadSemaphore = new SemaphoreSlim(5);

        public class JsonResult
        {
            public string Item1 { get; set; }
            public string Item2 { get; set; }
        }

        static async Task Main(string[] args)
        {
            if (args.Length != 2)
            {
                Console.WriteLine($"用法: {AppDomain.CurrentDomain.FriendlyName} [config*.json链接] [输出文件夹]");
                Console.ReadKey();
                return;
            }

            string ResPath = Path.GetDirectoryName(args[0]);
            string OutDir = args[1];
            string LogDir = Path.Combine(OutDir, LogName);
            if (File.Exists(LogDir)) { File.Delete(LogDir); }

            if (IsValidDownloadUrl(args[0]))
            {
                var jsonData = await DownloadJsonAsync(args[0]);

                if (jsonData != null)
                {
                    var paths = jsonData["paths"].ToObject<Dictionary<string, JArray>>();
                    var uuids = jsonData["uuids"].ToObject<List<string>>();
                    var nativeVersions = jsonData["versions"]["native"].ToObject<JArray>();
                    var importVersions = jsonData["versions"]["import"].ToObject<JArray>();
                    var types = jsonData["types"].ToObject<JArray>();

                    Console.WriteLine("解析文件...");

                    // 准备下载任务列表
                    var downloadTasks = new List<Task>();

                    foreach (var key in paths.Keys)
                    {
                        var pathArray = paths[key];
                        string pathString = pathArray[0].ToString();
                        int code1 = pathArray[1].ToObject<int>();

                        string uuid = uuids[int.Parse(key)];
                        string convertedUuid = Uuid2Uuid(uuid);
                        string uuidPrefix = convertedUuid.Substring(0, 2);

                        // 通过路径的序号获取 nativeString1 和 importString1
                        string nativeString1 = GetStringByIndex(nativeVersions, key);
                        string importString1 = GetStringByIndex(importVersions, key);


                        if (types[code1].ToString() == "cc.Asset")
                        {
                            string DownloadPath = ResPath.TrimEnd('/').Replace("\\", "/").Replace(":/", "://");
                            if (nativeString1 != null)
                            {
                                DownloadPath += $"/native/{uuidPrefix}/{convertedUuid}.{nativeString1}.atlas";
                            }
                            else
                            {
                                DownloadPath += $"/native/{uuidPrefix}/{convertedUuid}.atlas";
                            }
                            string OutPath = Path.Combine(OutDir, pathString + ".atlas");
                            downloadTasks.Add(DownloadAndSaveAsync(DownloadPath, OutPath, LogDir));
                        }
                        else if (types[code1].ToString() == "cc.ImageAsset")
                        {
                            string DownloadPath = ResPath.TrimEnd('/').Replace("\\", "/").Replace(":/", "://");
                            if (nativeString1 != null)
                            {
                                DownloadPath += $"/native/{uuidPrefix}/{convertedUuid}.{nativeString1}.png";
                            }
                            else
                            {
                                DownloadPath += $"/native/{uuidPrefix}/{convertedUuid}.png";
                            }
                            string OutPath = Path.Combine(OutDir, pathString + ".png");
                            downloadTasks.Add(DownloadAndSaveAsync(DownloadPath, OutPath, LogDir));
                        }
                        else if (types[code1].ToString() == "cc.TextAsset")
                        {
                            string DownloadPath = ResPath.TrimEnd('/').Replace("\\", "/").Replace(":/", "://");
                            if (importString1 != null)
                            {
                                DownloadPath += $"/import/{uuidPrefix}/{convertedUuid}.{importString1}.json";
                            }
                            else
                            {
                                DownloadPath += $"/import/{uuidPrefix}/{convertedUuid}.json";
                            }
                            string OutPath = Path.Combine(OutDir, pathString + ".json");
                            await DownloadAndSaveAsync(DownloadPath, OutPath, LogDir);
                            var (value1, value2) = ParseJsonData(OutPath);

                            //输出文件
                            string OutAtlas = Path.Combine(OutDir, pathString + ".atlas");
                            File.WriteAllText(OutAtlas, value2);
                            if (File.Exists(OutAtlas))
                            { File.Delete(OutPath); } 
                            Console.WriteLine($"解析成功：{OutPath}");

                        }
                        else if (types[code1].ToString() == "sp.SkeletonData")
                        {
                            string DownloadPath = ResPath.TrimEnd('/').Replace("\\", "/").Replace(":/", "://");
                            if (nativeString1 != null && !nativeString1.Contains("未找到md5"))
                            {
                                DownloadPath += $"/native/{uuidPrefix}/{convertedUuid}.{nativeString1}.bin";
                            }
                            else if(nativeString1.Contains("未找到md5"))
                            {
                                DownloadPath += $"/import/{uuidPrefix}/{convertedUuid}.{importString1}.json";
                            }
                            else
                            {
                                DownloadPath += $"/native/{uuidPrefix}/{convertedUuid}.bin";
                            }
                            if (nativeString1.Contains("未找到md5"))
                            {
                                string OutPath = Path.Combine(OutDir, pathString + ".json");
                                Directory.CreateDirectory(Path.GetDirectoryName(OutPath));

                                using (WebClient client = new WebClient())
                                {
                                    try
                                    {
                                        client.DownloadFile(DownloadPath, OutPath);
                                        Console.WriteLine($"下载成功：{OutPath}");

                                        try
                                        {
                                            string OutContent = File.ReadAllText(OutPath);
                                            JArray OutJArray = JArray.Parse(OutContent);
                                            if (OutJArray[3][0][0].ToString() == "sp.SkeletonData")
                                            {
                                                string name = OutJArray[5][0][1].ToString();
                                                string skeletonjson = OutJArray[5][0][4].ToString();
                                                File.WriteAllText(OutPath, skeletonjson);
                                            }
                                            Console.WriteLine($"解析成功：{OutPath}");
                                        }
                                        catch
                                        {
                                            File.AppendAllText(LogDir, $"解析失败：{OutPath}");
                                        }
                                    }
                                    catch
                                    {
                                        File.AppendAllText(LogDir, $"下载失败：{DownloadPath}\n输出路径为：{OutPath}\n\n");
                                    }
                                }

                            }
                            else
                            {
                                string OutPath = Path.Combine(OutDir, pathString + ".skel");
                                downloadTasks.Add(DownloadAndSaveAsync(DownloadPath, OutPath, LogDir));
                            }
                        }
                        else if (types[code1].ToString() == "cc.VideoClip")
                        {
                            string DownloadPath = ResPath.TrimEnd('/').Replace("\\", "/").Replace(":/", "://");
                            if (nativeString1 != null)
                            {
                                DownloadPath += $"/native/{uuidPrefix}/{convertedUuid}.{nativeString1}.mp4";
                            }
                            else
                            {
                                DownloadPath += $"/native/{uuidPrefix}/{convertedUuid}.mp4";
                            }
                            string OutPath = Path.Combine(OutDir, pathString + ".mp4");
                            downloadTasks.Add(DownloadAndSaveAsync(DownloadPath, OutPath, LogDir));

                        }
                    }

                    // 并行执行所有下载任务
                    await Task.WhenAll(downloadTasks);
                    Console.WriteLine("\n下载完成！");
                }
            }
        }


        static string Uuid2Uuid(string base64)
        {
            if (base64.Length != 22) return base64;

            var uuidTemplate = new StringBuilder("00000000000000000000000000000000");
            uuidTemplate[0] = base64[0];
            uuidTemplate[1] = base64[1];

            int i = 2, j = 2;

            while (i < 22)
            {
                int lhs = BASE64_KEYS.IndexOf(base64[i]);
                int rhs = BASE64_KEYS.IndexOf(base64[i + 1]);
                uuidTemplate[j] = HEX_CHARS[lhs >> 2];
                uuidTemplate[j + 1] = HEX_CHARS[((lhs & 3) << 2) | (rhs >> 4)];
                uuidTemplate[j + 2] = HEX_CHARS[rhs & 0xF];

                j += 3;
                i += 2;
            }

            uuidTemplate.Insert(8, '-');
            uuidTemplate.Insert(13, '-');
            uuidTemplate.Insert(18, '-');
            uuidTemplate.Insert(23, '-');

            return uuidTemplate.ToString();
        }

        static string GetStringByIndex(JArray versions, string index)
        {
            for (int i = 0; i < versions.Count; i += 2)
            {
                if (i + 1 < versions.Count) // 确保不超出范围
                {
                    int versionIndex;
                    if (int.TryParse(versions[i].ToString(), out versionIndex) && versionIndex.ToString() == index)
                    {
                        return versions[i + 1].ToString();
                    }
                }
            }
            return "未找到md5"; // 如果没有找到，返回"未找到md5"
        }


        // 异步下载并保存文件
        private static async Task DownloadAndSaveAsync(string url, string savePath, string errorLogPath)
        {
            string saveDir = Path.GetDirectoryName(savePath);

            try
            {
                // 等待信号量（控制并发）
                await _downloadSemaphore.WaitAsync();

                var response = await _httpClient.GetAsync(url);
                if (response.IsSuccessStatusCode)
                {
                    Directory.CreateDirectory(saveDir);
                    using (var fileStream = File.Create(savePath))
                    {
                        await response.Content.CopyToAsync(fileStream);
                    }
                    Console.WriteLine($"下载成功：{savePath}");
                }
                else
                {
                    lock (_fileLock)
                    {
                        File.AppendAllText(errorLogPath, $"下载失败：{url}\n输出路径为：{savePath}\n\n"); 
                    }
                }
            }
            catch
            {
                lock (_fileLock)
                {
                    File.AppendAllText(errorLogPath, $"下载失败：{url}\n输出路径为：{savePath}\n\n"); 
                }
            }
            finally
            {
                _downloadSemaphore.Release(); // 释放信号量
            }
        }

        //判断下载链接是否合法
        static bool IsValidDownloadUrl(string url)
        {
            if (string.IsNullOrWhiteSpace(url))
                return false;

            // 检查是否是合法的 Uri 格式（支持 http/https）
            bool isValidUri = Uri.TryCreate(url, UriKind.Absolute, out Uri uriResult)
                && (uriResult.Scheme == Uri.UriSchemeHttp || uriResult.Scheme == Uri.UriSchemeHttps);

            return isValidUri;
        }

        //下载json
        static async Task<JObject> DownloadJsonAsync(string url)
        {
            try
            {
                using (HttpClient client = new HttpClient())
                {
                    string jsonString = await client.GetStringAsync(url);
                    return JObject.Parse(jsonString);
                }
            }
            catch (HttpRequestException ex)
            {
                Console.WriteLine($"网络请求失败: {ex.Message}");
                return null;
            }
            catch (JsonException ex)
            {
                Console.WriteLine($"JSON解析失败: {ex.Message}");
                return null;
            }
        }


        static (string item1, string item2) ParseJsonData(string filePath)
        {
            try
            {
                string jsonContent = File.ReadAllText(filePath);
                JToken token = JToken.Parse(jsonContent);

                if (token is JArray jsonArray)
                {
                    // JArray处理逻辑
                    if (jsonArray.Count > 5 &&
                        jsonArray[5] is JArray outerArray &&
                        outerArray.Count > 0 &&
                        outerArray[0] is JArray innerArray &&
                        innerArray.Count > 2)
                    {
                        return (
                            innerArray[1]?.ToString(),
                            innerArray[2]?.ToString()
                        );
                    }
                    throw new InvalidDataException("JSON数组结构不符合预期");
                }
                else if (token is JObject jsonObject)
                {
                    // JObject处理逻辑
                    return (
                        jsonObject["_native"]?.ToString(),
                        null
                    );
                }
                else
                {
                    throw new InvalidDataException("未知的JSON类型");
                }
            }
            catch (Exception ex)
            {
                Console.WriteLine($"JSON解析失败: {ex.Message}");
                return (null, null);
            }
        }



    }
}
